@@ -112,12 +112,6 @@ public class AllManifestsTable extends BaseMetadataTable {
       return MetadataTableType.ALL_MANIFESTS.name();
     }
 
-    @Override
-    public long targetSplitSize() {
-      return tableOps().current().propertyAsLong(
-          TableProperties.METADATA_SPLIT_SIZE, TableProperties.METADATA_SPLIT_SIZE_DEFAULT);
-    }
-
     @Override
     protected CloseableIterable<FileScanTask> planFiles(
         TableOperations ops, Snapshot snapshot, Expression rowFilter,
