@@ -34,57 +34,6 @@ import org.slf4j.LoggerFactory;
 public abstract class BaseMetastoreCatalog implements Catalog {
   private static final Logger LOG = LoggerFactory.getLogger(BaseMetastoreCatalog.class);
 
-  @Override
-  public Table createTable(
-      TableIdentifier identifier,
-      Schema schema,
-      PartitionSpec spec,
-      String location,
-      Map<String, String> properties) {
-
-    return buildTable(identifier, schema)
-        .withPartitionSpec(spec)
-        .withLocation(location)
-        .withProperties(properties)
-        .create();
-  }
-
-  @Override
-  public Transaction newCreateTableTransaction(
-      TableIdentifier identifier,
-      Schema schema,
-      PartitionSpec spec,
-      String location,
-      Map<String, String> properties) {
-
-    return buildTable(identifier, schema)
-        .withPartitionSpec(spec)
-        .withLocation(location)
-        .withProperties(properties)
-        .createTransaction();
-  }
-
-  @Override
-  public Transaction newReplaceTableTransaction(
-      TableIdentifier identifier,
-      Schema schema,
-      PartitionSpec spec,
-      String location,
-      Map<String, String> properties,
-      boolean orCreate) {
-
-    TableBuilder tableBuilder = buildTable(identifier, schema)
-        .withPartitionSpec(spec)
-        .withLocation(location)
-        .withProperties(properties);
-
-    if (orCreate) {
-      return tableBuilder.createOrReplaceTransaction();
-    } else {
-      return tableBuilder.replaceTransaction();
-    }
-  }
-
   @Override
   public Table loadTable(TableIdentifier identifier) {
     Table result;
