@@ -27,6 +27,7 @@ import java.util.concurrent.atomic.AtomicLong;
 import java.util.function.Consumer;
 import org.apache.iceberg.encryption.EncryptionManager;
 import org.apache.iceberg.exceptions.CommitFailedException;
+import org.apache.iceberg.exceptions.CommitStateUnknownException;
 import org.apache.iceberg.exceptions.NoSuchTableException;
 import org.apache.iceberg.io.FileIO;
 import org.apache.iceberg.io.LocationProvider;
@@ -240,13 +241,13 @@ class BaseTransaction implements Transaction {
   }
 
   private void commitCreateTransaction() {
-    // fix up the snapshot log, which should not contain intermediate snapshots
-    TableMetadata createMetadata = current.removeSnapshotLogEntries(intermediateSnapshotIds);
-
     // this operation creates the table. if the commit fails, this cannot retry because another
     // process has created the same table.
     try {
-      ops.commit(null, createMetadata);
+      ops.commit(null, current);
+
+    } catch (CommitStateUnknownException e) {
+      throw e;
 
     } catch (RuntimeException e) {
       // the commit failed and no files were committed. clean up each update.
@@ -271,9 +272,7 @@ class BaseTransaction implements Transaction {
   }
 
   private void commitReplaceTransaction(boolean orCreate) {
-    // fix up the snapshot log, which should not contain intermediate snapshots
-    TableMetadata replaceMetadata = current.removeSnapshotLogEntries(intermediateSnapshotIds);
-    Map<String, String> props = base != null ? base.properties() : replaceMetadata.properties();
+    Map<String, String> props = base != null ? base.properties() : current.properties();
 
     try {
       Tasks.foreach(ops)
@@ -300,9 +299,12 @@ class BaseTransaction implements Transaction {
               this.base = underlyingOps.current(); // just refreshed
             }
 
-            underlyingOps.commit(base, replaceMetadata);
+            underlyingOps.commit(base, current);
           });
 
+    } catch (CommitStateUnknownException e) {
+      throw e;
+
     } catch (RuntimeException e) {
       // the commit failed and no files were committed. clean up each update.
       Tasks.foreach(updates)
@@ -358,9 +360,12 @@ class BaseTransaction implements Transaction {
             }
 
             // fix up the snapshot log, which should not contain intermediate snapshots
-            underlyingOps.commit(base, current.removeSnapshotLogEntries(intermediateSnapshotIds));
+            underlyingOps.commit(base, current);
           });
 
+    } catch (CommitStateUnknownException e) {
+      throw e;
+
     } catch (RuntimeException e) {
       // the commit failed and no files were committed. clean up each update.
       Tasks.foreach(updates)
@@ -454,6 +459,7 @@ class BaseTransaction implements Transaction {
     }
 
     @Override
+    @SuppressWarnings("ConsistentOverrides")
     public void commit(TableMetadata underlyingBase, TableMetadata metadata) {
       if (underlyingBase != current) {
         // trigger a refresh and retry
