@@ -62,7 +62,7 @@ class V1Metadata {
 
     @Override
     public void put(int i, Object v) {
-      throw new UnsupportedOperationException("Cannot read using IndexedManifestFile");
+      throw new UnsupportedOperationException("Cannot modify IndexedManifestFile wrapper via put");
     }
 
     @Override
@@ -214,7 +214,8 @@ class V1Metadata {
         DataFile.LOWER_BOUNDS,
         DataFile.UPPER_BOUNDS,
         DataFile.KEY_METADATA,
-        DataFile.SPLIT_OFFSETS
+        DataFile.SPLIT_OFFSETS,
+        DataFile.SORT_ORDER_ID
     );
   }
 
@@ -243,7 +244,7 @@ class V1Metadata {
 
     @Override
     public void put(int i, Object v) {
-      throw new UnsupportedOperationException("Cannot read using IndexedManifestEntry");
+      throw new UnsupportedOperationException("Cannot modify IndexedManifestEntry wrapper via put");
     }
 
     @Override
@@ -353,13 +354,15 @@ class V1Metadata {
           return wrapped.keyMetadata();
         case 13:
           return wrapped.splitOffsets();
+        case 14:
+          return wrapped.sortOrderId();
       }
       throw new IllegalArgumentException("Unknown field ordinal: " + pos);
     }
 
     @Override
     public void put(int i, Object v) {
-      throw new UnsupportedOperationException("Cannot read into IndexedDataFile");
+      throw new UnsupportedOperationException("Cannot modify IndexedDataFile wrapper via put");
     }
 
     @Override
@@ -447,6 +450,11 @@ class V1Metadata {
       return wrapped.splitOffsets();
     }
 
+    @Override
+    public Integer sortOrderId() {
+      return wrapped.sortOrderId();
+    }
+
     @Override
     public DataFile copy() {
       return wrapped.copy();
