@@ -19,7 +19,6 @@
 
 package org.apache.iceberg.io;
 
-import java.io.Closeable;
 import java.io.IOException;
 import java.nio.ByteBuffer;
 import org.apache.iceberg.DataFile;
@@ -31,7 +30,7 @@ import org.apache.iceberg.StructLike;
 import org.apache.iceberg.encryption.EncryptionKeyMetadata;
 import org.apache.iceberg.relocated.com.google.common.base.Preconditions;
 
-public class DataWriter<T> implements Closeable {
+public class DataWriter<T> implements FileWriter<T, DataWriteResult> {
   private final FileAppender<T> appender;
   private final FileFormat format;
   private final String location;
@@ -57,10 +56,22 @@ public class DataWriter<T> implements Closeable {
     this.sortOrder = sortOrder;
   }
 
+  @Override
+  public void write(T row) {
+    appender.add(row);
+  }
+
+  /**
+   * Writes a data record.
+   *
+   * @deprecated since 0.13.0, will be removed in 0.14.0; use {@link #write(Object)} instead.
+   */
+  @Deprecated
   public void add(T row) {
     appender.add(row);
   }
 
+  @Override
   public long length() {
     return appender.length();
   }
@@ -86,4 +97,9 @@ public class DataWriter<T> implements Closeable {
     Preconditions.checkState(dataFile != null, "Cannot create data file from unclosed writer");
     return dataFile;
   }
+
+  @Override
+  public DataWriteResult result() {
+    return new DataWriteResult(toDataFile());
+  }
 }
