@@ -22,6 +22,8 @@ package org.apache.iceberg.util;
 import java.util.function.Function;
 import org.apache.iceberg.BaseCombinedScanTask;
 import org.apache.iceberg.CombinedScanTask;
+import org.apache.iceberg.ContentFile;
+import org.apache.iceberg.FileContent;
 import org.apache.iceberg.FileScanTask;
 import org.apache.iceberg.io.CloseableIterable;
 import org.apache.iceberg.relocated.com.google.common.base.Preconditions;
@@ -36,6 +38,15 @@ public class TableScanUtil {
     return task.files().stream().anyMatch(TableScanUtil::hasDeletes);
   }
 
+  /**
+   * This is temporarily introduced since we plan to support pos-delete vectorized read first, then get to the
+   * equality-delete support. We will remove this method once both are supported.
+   */
+  public static boolean hasEqDeletes(CombinedScanTask task) {
+    return task.files().stream().anyMatch(
+        t -> t.deletes().stream().anyMatch(deleteFile -> deleteFile.content().equals(FileContent.EQUALITY_DELETES)));
+  }
+
   public static boolean hasDeletes(FileScanTask task) {
     return !task.deletes().isEmpty();
   }
@@ -56,7 +67,10 @@ public class TableScanUtil {
     Preconditions.checkArgument(lookback > 0, "Invalid split planning lookback (negative or 0): %s", lookback);
     Preconditions.checkArgument(openFileCost >= 0, "Invalid file open cost (negative): %s", openFileCost);
 
-    Function<FileScanTask, Long> weightFunc = file -> Math.max(file.length(), openFileCost);
+    // Check the size of delete file as well to avoid unbalanced bin-packing
+    Function<FileScanTask, Long> weightFunc = file -> Math.max(
+        file.length() + file.deletes().stream().mapToLong(ContentFile::fileSizeInBytes).sum(),
+        (1 + file.deletes().size()) * openFileCost);
 
     return CloseableIterable.transform(
         CloseableIterable.combine(
