@@ -19,8 +19,8 @@
 
 package org.apache.hadoop.hive.ql.exec.vector;
 
-import java.util.HashMap;
 import java.util.Map;
+import org.apache.iceberg.relocated.com.google.common.collect.Maps;
 
 /**
  * Copied here from Hive for compatibility
@@ -35,7 +35,7 @@ public class VectorizedSupport {
       this.lowerCaseName = name().toLowerCase();
     }
 
-    public static final Map<String, Support> nameToSupportMap = new HashMap<>();
+    public static final Map<String, Support> nameToSupportMap = Maps.newHashMap();
     static {
       for (Support support : values()) {
         nameToSupportMap.put(support.lowerCaseName, support);
