@@ -21,39 +21,59 @@ package org.apache.iceberg.nessie;
 
 import java.io.IOException;
 import java.nio.file.Path;
-import java.util.ArrayList;
 import java.util.List;
+import org.apache.avro.generic.GenericData.Record;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.iceberg.BaseTable;
 import org.apache.iceberg.CatalogProperties;
+import org.apache.iceberg.DataFile;
+import org.apache.iceberg.DataFiles;
+import org.apache.iceberg.Files;
 import org.apache.iceberg.Schema;
 import org.apache.iceberg.Table;
 import org.apache.iceberg.TableOperations;
+import org.apache.iceberg.avro.Avro;
 import org.apache.iceberg.catalog.TableIdentifier;
+import org.apache.iceberg.io.FileAppender;
 import org.apache.iceberg.relocated.com.google.common.collect.ImmutableMap;
+import org.apache.iceberg.relocated.com.google.common.collect.Lists;
 import org.apache.iceberg.types.Types;
 import org.apache.iceberg.types.Types.LongType;
 import org.apache.iceberg.types.Types.StructType;
 import org.junit.jupiter.api.AfterEach;
 import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.extension.ExtendWith;
 import org.junit.jupiter.api.extension.RegisterExtension;
 import org.junit.jupiter.api.io.TempDir;
-import org.projectnessie.api.ContentsApi;
-import org.projectnessie.api.TreeApi;
-import org.projectnessie.client.NessieClient;
+import org.projectnessie.client.api.NessieApiV1;
+import org.projectnessie.client.http.HttpClientBuilder;
 import org.projectnessie.error.NessieConflictException;
 import org.projectnessie.error.NessieNotFoundException;
-import org.projectnessie.jaxrs.NessieJaxRsExtension;
+import org.projectnessie.jaxrs.ext.NessieJaxRsExtension;
 import org.projectnessie.model.Branch;
 import org.projectnessie.model.Reference;
+import org.projectnessie.model.Tag;
+import org.projectnessie.versioned.persist.adapter.DatabaseAdapter;
+import org.projectnessie.versioned.persist.inmem.InmemoryDatabaseAdapterFactory;
+import org.projectnessie.versioned.persist.inmem.InmemoryTestConnectionProviderSource;
+import org.projectnessie.versioned.persist.tests.extension.DatabaseAdapterExtension;
+import org.projectnessie.versioned.persist.tests.extension.NessieDbAdapter;
+import org.projectnessie.versioned.persist.tests.extension.NessieDbAdapterName;
+import org.projectnessie.versioned.persist.tests.extension.NessieExternalDatabase;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import static org.apache.iceberg.types.Types.NestedField.required;
 
+@ExtendWith(DatabaseAdapterExtension.class)
+@NessieDbAdapterName(InmemoryDatabaseAdapterFactory.NAME)
+@NessieExternalDatabase(InmemoryTestConnectionProviderSource.class)
 public abstract class BaseTestIceberg {
+
+  @NessieDbAdapter
+  static DatabaseAdapter databaseAdapter;
   @RegisterExtension
-  static NessieJaxRsExtension server = new NessieJaxRsExtension();
+  static NessieJaxRsExtension server = new NessieJaxRsExtension(() -> databaseAdapter);
 
   private static final Logger LOGGER = LoggerFactory.getLogger(BaseTestIceberg.class);
 
@@ -61,42 +81,37 @@ public abstract class BaseTestIceberg {
   public Path temp;
 
   protected NessieCatalog catalog;
-  protected NessieClient client;
-  protected TreeApi tree;
-  protected ContentsApi contents;
+  protected NessieApiV1 api;
   protected Configuration hadoopConfig;
   protected final String branch;
-  private String uri;
+  protected String uri;
 
   public BaseTestIceberg(String branch) {
     this.branch = branch;
   }
 
   private void resetData() throws NessieConflictException, NessieNotFoundException {
-    for (Reference r : tree.getAllReferences()) {
+    for (Reference r : api.getAllReferences().get().getReferences()) {
       if (r instanceof Branch) {
-        tree.deleteBranch(r.getName(), r.getHash());
+        api.deleteBranch().branch((Branch) r).delete();
       } else {
-        tree.deleteTag(r.getName(), r.getHash());
+        api.deleteTag().tag((Tag) r).delete();
       }
     }
-    tree.createReference(Branch.of("main", null));
+    api.createReference().reference(Branch.of("main", null)).create();
   }
 
   @BeforeEach
   public void beforeEach() throws IOException {
-    String port = System.getProperty("quarkus.http.test-port", "19120");
     uri = server.getURI().toString();
-    this.client = NessieClient.builder().withUri(uri).build();
-    tree = client.getTreeApi();
-    contents = client.getContentsApi();
+    this.api = HttpClientBuilder.builder().withUri(uri).build(NessieApiV1.class);
 
     resetData();
 
     try {
-      tree.createReference(Branch.of(branch, null));
+      api.createReference().reference(Branch.of(branch, null)).create();
     } catch (Exception e) {
-      // ignore, already created. Cant run this in BeforeAll as quarkus hasn't disabled auth
+      // ignore, already created. Can't run this in BeforeAll as quarkus hasn't disabled auth
     }
 
     hadoopConfig = new Configuration();
@@ -129,24 +144,35 @@ public abstract class BaseTestIceberg {
   }
 
   protected static Schema schema(int count) {
-    List<Types.NestedField> fields = new ArrayList<>();
+    List<Types.NestedField> fields = Lists.newArrayList();
     for (int i = 0; i < count; i++) {
       fields.add(required(i, "id" + i, Types.LongType.get()));
     }
     return new Schema(Types.StructType.of(fields).fields());
   }
 
-  void createBranch(String name, String hash) throws NessieNotFoundException, NessieConflictException {
-    tree.createReference(Branch.of(name, hash));
+  void createBranch(String name, String hash)
+      throws NessieNotFoundException, NessieConflictException {
+    createBranch(name, hash, "main");
+  }
+
+  void createBranch(String name, String hash, String sourceRef)
+      throws NessieNotFoundException, NessieConflictException {
+    api.createReference().reference(Branch.of(name, hash)).sourceRefName(sourceRef).create();
   }
 
   @AfterEach
   public void afterEach() throws Exception {
-    catalog.close();
-    client.close();
-    catalog = null;
-    client = null;
-    hadoopConfig = null;
+    try {
+      if (catalog != null) {
+        catalog.close();
+      }
+      api.close();
+    } finally {
+      catalog = null;
+      api = null;
+      hadoopConfig = null;
+    }
   }
 
   static String metadataLocation(NessieCatalog catalog, TableIdentifier tableIdentifier) {
@@ -156,4 +182,28 @@ public abstract class BaseTestIceberg {
     NessieTableOperations icebergOps = (NessieTableOperations) ops;
     return icebergOps.currentMetadataLocation();
   }
+
+  static String writeRecordsToFile(Table table, Schema schema, String filename,
+      List<Record> records)
+      throws IOException {
+    String fileLocation = table.location().replace("file:", "") +
+        String.format("/data/%s.avro", filename);
+    try (FileAppender<Record> writer = Avro.write(Files.localOutput(fileLocation))
+        .schema(schema)
+        .named("test")
+        .build()) {
+      for (Record rec : records) {
+        writer.add(rec);
+      }
+    }
+    return fileLocation;
+  }
+
+  static DataFile makeDataFile(Table icebergTable, String fileLocation) {
+    return DataFiles.builder(icebergTable.spec())
+        .withRecordCount(3)
+        .withPath(fileLocation)
+        .withFileSizeInBytes(Files.localInput(fileLocation).getLength())
+        .build();
+  }
 }
