@@ -19,6 +19,7 @@
 
 package org.apache.iceberg.nessie;
 
+import java.util.Collections;
 import java.util.List;
 import org.apache.iceberg.catalog.Namespace;
 import org.apache.iceberg.catalog.TableIdentifier;
@@ -59,4 +60,26 @@ public class TestNamespace extends BaseTestIceberg {
     namespaces = catalog.listNamespaces(Namespace.of("b"));
     Assertions.assertThat(namespaces).isNotNull().hasSize(2);
   }
+
+  @Test
+  public void testDroppingNamespace() {
+    Assertions.assertThatThrownBy(() -> catalog.dropNamespace(Namespace.of("test")))
+        .isInstanceOf(UnsupportedOperationException.class)
+        .hasMessage("Cannot drop namespace 'test': dropNamespace is not supported by the NessieCatalog");
+  }
+
+  @Test
+  public void testSettingProperties() {
+    Assertions.assertThatThrownBy(() -> catalog.setProperties(Namespace.of("test"), Collections.emptyMap()))
+        .isInstanceOf(UnsupportedOperationException.class)
+        .hasMessage("Cannot set properties for namespace 'test': setProperties is not supported by the NessieCatalog");
+  }
+
+  @Test
+  public void testRemovingProperties() {
+    Assertions.assertThatThrownBy(() -> catalog.removeProperties(Namespace.of("test"), Collections.emptySet()))
+        .isInstanceOf(UnsupportedOperationException.class)
+        .hasMessage(
+            "Cannot remove properties for namespace 'test': removeProperties is not supported by the NessieCatalog");
+  }
 }
